import warnings

from pdm.pep517._vendor.license_expression import (
    LicenseSymbol,
    Licensing,
    get_spdx_licensing,
)
from pdm.pep517.exceptions import MetadataError, PDMWarning


def get_licensing() -> Licensing:
    """Get the spdx licensing with two additional license symbols."""
    extra_identifiers = ["LicenseRef-Public-Domain", "LicenseRef-Proprietary"]
    licensing = get_spdx_licensing()
    for identifier in extra_identifiers:
        symbol = LicenseSymbol(identifier)
        licensing.known_symbols[symbol.key] = symbol
        licensing.known_symbols_lowercase[symbol.key.lower()] = symbol
    return licensing


_licensing = get_licensing()


def normalize_expression(expression: str) -> str:
    """Normalize a SPDX license expression."""
    validate_result = _licensing.validate(expression)
    if validate_result.errors:
        raise MetadataError("license", validate_result.errors)
    result = validate_result.normalized_expression or expression
    if result != expression:
        warnings.warn(
            f"License expression normalized to '{result}'", PDMWarning, stacklevel=2
        )
    return result



"""
    Extensible validation for Python dictionaries.

    :copyright: 2012-2021 by Nicola Iarocci.
    :license: ISC, see LICENSE for more details.

    Full documentation is available at https://python-cerberus.org/

"""

from __future__ import absolute_import

from pdm.pep517._vendor.cerberus.validator import DocumentError, Validator
from pdm.pep517._vendor.cerberus.schema import rules_set_registry, schema_registry, SchemaError
from pdm.pep517._vendor.cerberus.utils import TypeDefinition


__version__ = "unknown"

__all__ = [
    DocumentError.__name__,
    SchemaError.__name__,
    TypeDefinition.__name__,
    Validator.__name__,
    "schema_registry",
    "rules_set_registry",
]



from __future__ import annotations

from typing import Any


class BuildError(RuntimeError):
    pass


class ProjectError(ValueError):
    pass


class MetadataError(ProjectError):
    def __init__(self, field: str, error: Any) -> None:
        message = f"{field}: {error}"
        super().__init__(message)


class PDMWarning(UserWarning):
    pass


class PEP621ValidationError(ProjectError):
    def __init__(self, errors: list[str]) -> None:
        super().__init__(errors)
        self.errors = errors
